#pragma once
#ifndef PLAYER_ORBIT_ES_H
#define PLAYER_ORBIT_ES_H

#include "sgg/graphics.h"
#include <cmath>
#define M_PI 3.14159265358979323846

/**
 * Estructura que define las propiedades de un jugador, representado como un círculo.
 */
struct Player {
    float x, y;       ///< Posición central del jugador.
    float radius;     ///< Radio del círculo que representa al jugador.
    float speed;      ///< Velocidad de movimiento del jugador.
    float angle;      ///< Ángulo en grados para movimientos direccionales, como lanzamientos inclinados.
    float time;       ///< Variable de tiempo para movimientos basados en tiempo (por ejemplo, caída libre).
};

/**
 * Mueve el jugador horizontalmente a una velocidad especificada.
 * @param player Referencia a la estructura Player que será movida.
 * @param speed Velocidad horizontal que se aplicará al movimiento del jugador.
 */
void moveHorizontal(Player& player, float speed) {
    player.x += speed;
}

/**
 * Mueve el jugador verticalmente a una velocidad especificada.
 * @param player Referencia a la estructura Player que será movida.
 * @param speed Velocidad vertical que se aplicará al movimiento del jugador.
 */
void moveVertical(Player& player, float speed) {
    player.y += speed;
}

/**
 * Mueve el jugador en un movimiento circular alrededor de un centro especificado.
 * @param player Referencia a la estructura Player que será movida.
 * @param centerX Coordenada X del centro de la trayectoria circular.
 * @param centerY Coordenada Y del centro de la trayectoria circular.
 * @param radius Radio de la trayectoria circular.
 * @param speed Velocidad angular del movimiento circular en grados por actualización.
 */
void moveCircular(Player& player, float centerX, float centerY, float radius, float speed) {
    player.angle += speed;
    player.x = centerX + radius * cos(player.angle * M_PI / 180.0f);
    player.y = centerY + radius * sin(player.angle * M_PI / 180.0f);
}

/**
 * Simula el movimiento de caída libre para el jugador bajo la influencia de la gravedad.
 * @param player Referencia a la estructura Player que será movida.
 * @param gravity Aceleración debido a la gravedad (por defecto 9.8 m/s^2).
 */
void freeFall(Player& player, float gravity = 9.8f) {
    player.y += 0.5f * gravity * player.time * player.time;
    player.time += 0.1f; // Actualiza el tiempo en cada fotograma
}

/**
 * Simula un lanzamiento vertical hacia arriba para el jugador.
 * @param player Referencia a la estructura Player que será movida.
 * @param initialVelocity Velocidad vertical inicial del lanzamiento.
 * @param gravity Aceleración debido a la gravedad (por defecto 9.8 m/s^2).
 */
void verticalThrow(Player& player, float initialVelocity, float gravity = 9.8f) {
    player.y -= initialVelocity * player.time - 0.5f * gravity * player.time * player.time;
    player.time += 0.1f;
}

/**
 * Simula un lanzamiento inclinado en un ángulo para el jugador.
 * @param player Referencia a la estructura Player que será movida.
 * @param initialVelocity Velocidad inicial del lanzamiento.
 * @param angle Ángulo del lanzamiento en grados.
 * @param gravity Aceleración debido a la gravedad (por defecto 9.8 m/s^2).
 */
void inclinedThrow(Player& player, float initialVelocity, float angle, float gravity = 9.8f) {
    float radAngle = angle * M_PI / 180.0f;
    player.x += initialVelocity * cos(radAngle) * player.time;
    player.y -= (initialVelocity * sin(radAngle) * player.time - 0.5f * gravity * player.time * player.time);
    player.time += 0.1f;
}

/**
 * Mueve el jugador en una trayectoria elíptica con velocidad constante.
 * @param player Referencia a la estructura Player que será movida.
 * @param centerX Coordenada X del centro de la trayectoria elíptica.
 * @param centerY Coordenada Y del centro de la trayectoria elíptica.
 * @param a Longitud del semi-eje mayor de la elipse.
 * @param b Longitud del semi-eje menor de la elipse.
 * @param speed Velocidad angular del movimiento elíptico en grados por actualización.
 */
void ellipticalMotion(Player& player, float centerX, float centerY, float a, float b, float speed) {
    player.angle += speed;
    player.x = centerX + a * cos(player.angle * M_PI / 180.0f);
    player.y = centerY + b * sin(player.angle * M_PI / 180.0f);
}

/**
 * Dibuja el jugador como un círculo en la posición actual.
 * @param player Referencia constante a la estructura Player que será dibujada.
 */
void drawPlayer(const Player& player) {
    graphics::Brush brush;
    brush.fill_color[0] = 0.0f; // Color azul para el jugador
    brush.fill_color[1] = 0.0f;
    brush.fill_color[2] = 1.0f;
    graphics::drawDisk(player.x, player.y, player.radius, brush);
}

/**
 * Restaura la variable de tiempo del jugador para los cálculos de movimiento.
 * @param player Referencia a la estructura Player cuyo tiempo será restaurado.
 */
void resetTime(Player& player) {
    player.time = 0.0f;
}

#endif // PLAYER_ORBIT_ES_H


